unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ExtCtrls, Buttons, AppEvnts, mxOneInstance;

type
  TFBServers = (fbNone, fb15, fb20, fbMulti);

  TConfig = record
    FB15CSCommand: string;
    FB15SSCommand: string;
    FB15StopCommand: string;
    FB20CSCommand: string;
    FB20SSCommand: string;
    FB20StopCommand: string;
    StartupIndex: Integer;
    AllowMultiFB: Boolean;
    TrayActionIndex: Integer;
  end;

  TFormConfig = class(TForm)
    TrayIcon: TTrayIcon;
    TrayPopMenu: TPopupMenu;
    AppEvents: TApplicationEvents;
    OneInstance: TmxOneInstance;
    OpenDialog: TOpenDialog;

    ItemName: TMenuItem;
    ItemStatus: TMenuItem;
    Separator1: TMenuItem;
    MenuFB15: TMenuItem;
      ItemStartFB15CS: TMenuItem;
      ItemStartFB15SS: TMenuItem;
      Separator3: TMenuItem;
      ItemStopFB15: TMenuItem;
    MenuFB20: TMenuItem;
      ItemStartFB20CS: TMenuItem;
      ItemStartFB20SS: TMenuItem;
      Separator4: TMenuItem;
      ItemStopFB20: TMenuItem;
    Separator2: TMenuItem;
    ItemConfig: TMenuItem;
    ItemExit: TMenuItem;

    lblFB15Title: TLabel;
    bvlFB15: TBevel;
    lblFB15CS: TLabel;
    edFB15CS: TEdit;
    btnFB15CS: TSpeedButton;
    lblFB15SS: TLabel;
    edFB15SS: TEdit;
    btnFB15SS: TSpeedButton;
    lblFB15Stop: TLabel;
    edFB15Stop: TEdit;
    btnFB15Stop: TSpeedButton;

    lblFB20Title: TLabel;
    bvlFB20: TBevel;
    lblFB20CS: TLabel;
    edFB20CS: TEdit;
    btnFB20CS: TSpeedButton;
    lblFB20SS: TLabel;
    edFB20SS: TEdit;
    btnFB20SS: TSpeedButton;
    lblFB20Stop: TLabel;
    edFB20Stop: TEdit;
    btnFB20Stop: TSpeedButton;
    lblOptions: TLabel;
    bvlOptions: TBevel;
    lblMultiServer: TLabel;
    cbMultiServer: TCheckBox;
    lblStartup: TLabel;
    cbStartup: TComboBox;
    lblTrayDouble: TLabel;
    cbTrayDouble: TComboBox;

    btnSave: TButton;
    btnCancel: TButton;

    procedure ItemStartFB15CSClick(Sender: TObject);
    procedure ItemStartFB15SSClick(Sender: TObject);
    procedure ItemStartFB20CSClick(Sender: TObject);
    procedure ItemStartFB20SSClick(Sender: TObject);
    procedure ItemStopFB15Click(Sender: TObject);
    procedure ItemStopFB20Click(Sender: TObject);
    procedure ItemConfigClick(Sender: TObject);
    procedure ItemExitClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AppEventsMinimize(Sender: TObject);
    procedure AppEventsRestore(Sender: TObject);
    procedure TrayIconDblClick(Sender: TObject);
    procedure OneInstanceInstanceExists(Sender: TObject);
    procedure btnOpenCommandClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
  private
    RunningServer: TFBServers;
    ConfigFile: string;
    Config: TConfig;

    procedure CreateDefaultConfig(configFile: string);
    procedure ReadConfigValues(configFile: string);
    procedure ApplyConfigValues;
  public
    { Public declarations }
  end;

var
  FormConfig: TFormConfig;

implementation

{$R *.dfm}

uses
  ShellAPI, INIFiles;

procedure TFormConfig.CreateDefaultConfig(configFile: string);
var
  configText: TStringList;
begin
  configText := TStringList.Create;
  configText.Clear;
  configText.Add('[VERSION]');
  configText.Add('Major=0');
  configText.Add('Minor=1');
  configText.Add('');
  configText.Add('[COMMANDS]');
  configText.Add('FB15CS="C:\Applications\Firebird\bin\install_classic.bat"');
  configText.Add('FB15SS="C:\Applications\Firebird\bin\install_super.bat"');
  configText.Add('FB15Stop="C:\Applications\Firebird\bin\uninstall.bat"');
  configText.Add('FB20CS="C:\Applications\Firebird2\bin\install_classic.bat"');
  configText.Add('FB20SS="C:\Applications\Firebird2\bin\install_super.bat"');
  configText.Add('FB20Stop="C:\Applications\Firebird2\bin\uninstall.bat"');
  configText.Add('');
  configText.Add('[OPTIONS]');
  configText.Add('MultiFB=0');
  configText.Add('Startup=0');
  configText.Add('TrayDouble=0');
  configText.SaveToFile(configFile);
  configText.Free;
end;

procedure TFormConfig.ReadConfigValues(configFile: string);
var
  configText: TIniFile;
begin
  configText := TIniFile.Create(configFile);
  Config.FB15CSCommand := configText.ReadString('COMMANDS', 'FB15CS', '');
  Config.FB15SSCommand := configText.ReadString('COMMANDS', 'FB15SS', '');
  Config.FB15StopCommand := configText.ReadString('COMMANDS', 'FB15Stop', '');
  Config.FB20CSCommand := configText.ReadString('COMMANDS', 'FB20CS', '');
  Config.FB20SSCommand := configText.ReadString('COMMANDS', 'FB20SS', '');
  Config.FB20StopCommand := configText.ReadString('COMMANDS', 'FB20Stop', '');
  Config.TrayActionIndex := configText.ReadInteger('OPTIONS', 'TrayDouble', 0);
  Config.StartupIndex := configText.ReadInteger('OPTIONS', 'Startup', 0);
  Config.AllowMultiFB := configText.ReadBool('OPTIONS', 'MultiFB', False);
  configText.Free;
end;

procedure TFormConfig.ApplyConfigValues;
begin
  edFB15CS.Text := Config.FB15CSCommand;
  edFB15SS.Text := Config.FB15SSCommand;
  edFB15Stop.Text := Config.FB15StopCommand;
  edFB20CS.Text := Config.FB20CSCommand;
  edFB20SS.Text := Config.FB20SSCommand;
  edFB20Stop.Text := Config.FB20StopCommand;
  cbTrayDouble.ItemIndex := Config.TrayActionIndex;
  cbStartup.ItemIndex := Config.StartupIndex;
  cbMultiServer.Checked := Config.AllowMultiFB;
end;

procedure TFormConfig.AppEventsMinimize(Sender: TObject);
begin
  ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TFormConfig.AppEventsRestore(Sender: TObject);
begin
  ShowWindow(Application.Handle, SW_SHOW);
end;

procedure TFormConfig.FormCreate(Sender: TObject);
begin
  // open and read config file
  ConfigFile := ChangeFileExt(Application.ExeName,'.ini');
  if not FileExists(ConfigFile) then CreateDefaultConfig(ConfigFile);
  ReadConfigValues(ConfigFile);
  ApplyConfigValues;

  // startup server
  case cbStartup.ItemIndex of
    0: // run nothing
       RunningServer := fbNone;
    1: // run FB 1.5 Classic
       ItemStartFB15CSClick(self);
    2: // run FB 1.5 Super
       ItemStartFB15SSClick(self);
    3: // run FB 2.0 Classic
       ItemStartFB20CSClick(self);
    4: // run FB 2.0 Super
       ItemStartFB20SSClick(self);
  end;
end;

procedure TFormConfig.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
  Application.Minimize;
end;

procedure TFormConfig.btnOpenCommandClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    case (Sender as TComponent).Tag of
      1: edFB15CS.Text := OpenDialog.FileName;
      2: edFB15SS.Text := OpenDialog.FileName;
      3: edFB15Stop.Text := OpenDialog.FileName;
      4: edFB20CS.Text := OpenDialog.FileName;
      5: edFB20CS.Text := OpenDialog.FileName;
      6: edFB20Stop.Text := OpenDialog.FileName;
    end;
end;

procedure TFormConfig.btnCancelClick(Sender: TObject);
begin
  // reset to last config values
  ApplyConfigValues;
  Application.Minimize;
end;

procedure TFormConfig.btnSaveClick(Sender: TObject);
var
  configText: TIniFile;
begin
  // save new config values
  configText := TIniFile.Create(configFile);
  configText.WriteString('COMMANDS', 'FB15CS', '"'+edFB15CS.Text+'"');
  configText.WriteString('COMMANDS', 'FB15SS', '"'+edFB15SS.Text+'"');
  configText.WriteString('COMMANDS', 'FB15Stop', '"'+edFB15Stop.Text+'"');
  configText.WriteString('COMMANDS', 'FB20CS', '"'+edFB20CS.Text+'"');
  configText.WriteString('COMMANDS', 'FB20SS', '"'+edFB20CS.Text+'"');
  configText.WriteString('COMMANDS', 'FB20Stop', '"'+edFB20Stop.Text+'"');
  configText.WriteInteger('OPTIONS', 'TrayDouble', cbTrayDouble.ItemIndex);
  configText.WriteInteger('OPTIONS', 'Startup', cbStartup.ItemIndex);
  configText.WriteBool('OPTIONS', 'MultiFB', cbMultiServer.Checked);
  configText.Free;
  Application.Minimize;
end;

procedure TFormConfig.ItemExitClick(Sender: TObject);
begin
  if Application.MessageBox('Are you sure want to exit?','Exit',MB_YESNO or MB_ICONQUESTION) = mrYes then
  begin
    // stop server(s)
    if RunningServer = fb15 then
      ItemStopFB15Click(self)
    else if RunningServer = fb20 then
      ItemStopFB20Click(self)
    else if RunningServer = fbMulti then
    begin
      ItemStopFB15Click(self);
      ItemStopFB20Click(self);
    end;
    // stop applications
    Application.Terminate;
  end;
end;

procedure TFormConfig.ItemConfigClick(Sender: TObject);
begin
  // read saved config values
  ReadConfigValues(ConfigFile);
  ApplyConfigValues;
  // show config window
  Application.Restore;
  Application.BringToFront;
end;

procedure TFormConfig.ItemStartFB15CSClick(Sender: TObject);
begin
  // refuse to run multiple servers
  if (not cbMultiServer.Checked) and (RunningServer <> fbNone) then
  begin
    ShowMessage('Another Firebird server is already running.');
    Exit;
  end;

  // run selected FB server
  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB15CS.Text), nil,
                  PChar(ExtractFileDir(edFB15CS.Text)), SW_SHOWNORMAL) > 32 then
  begin
    // update menu indicators
    ItemStartFB15CS.Checked := True;
    ItemStartFB15SS.Checked := False;
    ItemStartFB15CS.Enabled := False;
    ItemStartFB15SS.Enabled := False;
    ItemStopFB15.Enabled := True;

    // update status texts
    if RunningServer = fbNone then
    begin
      RunningServer := fb15;
      ItemStatus.Caption := 'Status: FB 1.5 Classic is running';
      TrayIcon.Hint := 'FB Multistarter - FB 1.5 Classic is running';
    end
    else if RunningServer = fb20 then
    begin
      RunningServer := fbMulti;
      ItemStatus.Caption := 'Status: FB 1.5 and 2.0 are running';
      TrayIcon.Hint := 'FB Multistarter - FB 1.5 and 2.0 are running';
    end;
  end;
end;

procedure TFormConfig.ItemStartFB15SSClick(Sender: TObject);
begin
  if (not cbMultiServer.Checked) and (RunningServer <> fbNone) then
  begin
    ShowMessage('Another Firebird server is already running.');
    Exit;
  end;

  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB15SS.Text), nil,
                  PChar(ExtractFileDir(edFB15SS.Text)), SW_SHOWNORMAL) > 32 then
  begin
    ItemStartFB15CS.Checked := False;
    ItemStartFB15SS.Checked := True;
    ItemStartFB15CS.Enabled := False;
    ItemStartFB15SS.Enabled := False;
    ItemStopFB15.Enabled := True;

    if RunningServer = fbNone then
    begin
      RunningServer := fb15;
      ItemStatus.Caption := 'Status: FB 1.5 Super is running';
      TrayIcon.Hint := 'FB Multistarter - FB 1.5 Super is running';
    end
    else if RunningServer = fb20 then
    begin
      RunningServer := fbMulti;
      ItemStatus.Caption := 'Status: FB 1.5 and 2.0 are running';
      TrayIcon.Hint := 'FB Multistarter - FB 1.5 and 2.0 are running';
    end;
  end;
end;

procedure TFormConfig.ItemStartFB20CSClick(Sender: TObject);
begin
  if (not cbMultiServer.Checked) and (RunningServer <> fbNone) then
  begin
    ShowMessage('Another Firebird server is already running.');
    Exit;
  end;

  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB20CS.Text), nil,
                  PChar(ExtractFileDir(edFB20CS.Text)), SW_SHOWNORMAL) > 32 then
  begin
    ItemStartFB20CS.Checked := True;
    ItemStartFB20SS.Checked := False;
    ItemStartFB20CS.Enabled := False;
    ItemStartFB20SS.Enabled := False;
    ItemStopFB20.Enabled := True;

    if RunningServer = fbNone then
    begin
      RunningServer := fb20;
      ItemStatus.Caption := 'Status: FB 2.0 Classic is running';
      TrayIcon.Hint := 'FB Multistarter - FB 2.0 Classic is running';
    end
    else if RunningServer = fb15 then
    begin
      RunningServer := fbMulti;
      ItemStatus.Caption := 'Status: FB 2.0 and 1.5 are running';
      TrayIcon.Hint := 'FB Multistarter - FB 2.0 and 1.5 are running';
    end;
  end;
end;

procedure TFormConfig.ItemStartFB20SSClick(Sender: TObject);
begin
  if (not cbMultiServer.Checked) and (RunningServer <> fbNone) then
  begin
    ShowMessage('Another Firebird server is already running.');
    Exit;
  end;

  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB20SS.Text), nil,
                  PChar(ExtractFileDir(edFB20SS.Text)), SW_SHOWNORMAL) > 32 then
  begin
    ItemStartFB20CS.Checked := False;
    ItemStartFB20SS.Checked := True;
    ItemStartFB20CS.Enabled := False;
    ItemStartFB20SS.Enabled := False;
    ItemStopFB20.Enabled := True;

    if RunningServer = fbNone then
    begin
      RunningServer := fb20;
      ItemStatus.Caption := 'Status: FB 2.0 Super is running';
      TrayIcon.Hint := 'FB Multistarter - FB 2.0 Super is running';
    end
    else if RunningServer = fb15 then
    begin
      RunningServer := fbMulti;
      ItemStatus.Caption := 'Status: FB 2.0 and 1.5 are running';
      TrayIcon.Hint := 'FB Multistarter - FB 2.0 and 1.5 are running';
    end;
  end;
end;

procedure TFormConfig.ItemStopFB15Click(Sender: TObject);
begin
  // stop selected FB server
  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB15Stop.Text), nil,
                  PChar(ExtractFileDir(edFB15Stop.Text)), SW_SHOWNORMAL) > 32 then
  begin
    // update menu indicators
    ItemStartFB15CS.Checked := False;
    ItemStartFB15SS.Checked := False;
    ItemStartFB15CS.Enabled := True;
    ItemStartFB15SS.Enabled := True;
    ItemStopFB15.Enabled := False;

    // update status texts
    if RunningServer = fbMulti then
    begin
      RunningServer := fb20;
      // check remaining server running
      if ItemStartFB20CS.Checked then
      begin
        ItemStatus.Caption := 'Status: FB 2.0 Classic is running';
        TrayIcon.Hint := 'FB Multistarter - FB 2.0 Classic is running';
      end
      else
      begin
        ItemStatus.Caption := 'Status: FB 2.0 Super is running';
        TrayIcon.Hint := 'FB Multistarter - FB 2.0 Super is running';
      end
    end
    else if RunningServer = fb15 then
    begin
      RunningServer := fbNone;
      ItemStatus.Caption := 'Status: All servers are down.';
      TrayIcon.Hint := 'FB Multistarter - All servers are down.';
    end;
  end;
end;

procedure TFormConfig.ItemStopFB20Click(Sender: TObject);
begin
  if ShellExecute(Application.Handle, 'open',
                  PChar(edFB20Stop.Text), nil,
                  PChar(ExtractFileDir(edFB20Stop.Text)), SW_SHOWNORMAL) > 32 then
  begin
    ItemStartFB20CS.Checked := False;
    ItemStartFB20SS.Checked := False;
    ItemStartFB20CS.Enabled := True;
    ItemStartFB20SS.Enabled := True;
    ItemStopFB20.Enabled := False;

    if RunningServer = fbMulti then
    begin
      RunningServer := fb15;
      if ItemStartFB15CS.Checked then
      begin
        ItemStatus.Caption := 'Status: FB 1.5 Classic is running';
        TrayIcon.Hint := 'FB Multistarter - FB 1.5 Classic is running';
      end
      else
      begin
        ItemStatus.Caption := 'Status: FB 1.5 Super is running';
        TrayIcon.Hint := 'FB Multistarter - FB 1.5 Super is running';
      end
    end
    else if RunningServer = fb20 then
    begin
      RunningServer := fbNone;
      ItemStatus.Caption := 'Status: All servers are down.';
      TrayIcon.Hint := 'FB Multistarter - All servers are down.';
    end;
  end;
end;

procedure TFormConfig.TrayIconDblClick(Sender: TObject);
begin
  case cbTrayDouble.ItemIndex of
    0: // open config
       ItemConfigClick(TrayIcon);
    1: // toggle FB 1.5 Classic
       if RunningServer = fbNone then
         ItemStartFB15CSClick(TrayIcon)
       else
         ItemStopFB15Click(TrayIcon);
    2: // toggle FB 1.5 Super
       if RunningServer = fbNone then
         ItemStartFB15SSClick(TrayIcon)
       else
         ItemStopFB15Click(TrayIcon);
    3: // toggle FB 2.0 Classic
       if RunningServer = fbNone then
         ItemStartFB20CSClick(TrayIcon)
       else
         ItemStopFB20Click(TrayIcon);
    4: // toggle FB 2.0 Super
       if RunningServer = fbNone then
         ItemStartFB20SSClick(TrayIcon)
       else
         ItemStopFB20Click(TrayIcon);
    5: ; // nothing
  end;
end;

procedure TFormConfig.OneInstanceInstanceExists(Sender: TObject);
begin
  ShowMessage('Firebird Multistarter is already running.');
end;

end.

